<?php
//////////////////////////////////////////////////////////////////////////
//
// Class to query the eBay API
//  Base class for all ebay api calls.
//
//////////////////////////////////////////////////////////////////////////

require_once __DIR__."/constants.php";
require_once __DIR__."/utilities.php";
require_once __DIR__."/tracking.php";
  
abstract class fuEbayApiCall
{
  use fuEbayTracking; 

  private const FEL_ENDPOINT = "https://felp.fubaby.com/";
  private const FEL_CACHEEXPIRY = 21600;

  protected static $id = 0;
  protected $globalId;
  protected $deferLoading;
  protected $disableApiCache;
  protected $callCacheExpiry = FUBABY_EBAY_CALLCACHEEXPIRY;
  protected $requestUri;
  protected $result = "";
  protected $resultCount = 0;
  
  public $enableTraceLog = false;
  public $displayFields = [];

  // Core properties to adjust presentation.
  public $customId = '';
  public $picWidth = 0;
  public $arrangement = 0;
  
  function __construct() 
  {
    $this->globalId = get_option('fubaby_ebayGlobalID');
    $this->deferLoading = get_option('fubaby_ebayDeferredLoading', fuEbayDeferedLoading::NotCached);
    $this->disableApiCache = get_option('fubaby_ebayDisableAPICache', false);
    $this->enableTraceLog = get_option('fubaby_ebayTraceLogging', false);
  }

  protected function initEbayListing($item)
  {}

  abstract protected function createListingGrid($resp);

  abstract protected function createDeferredLoadingStub();
  
  // Utility funcs.
  protected function traceLog(string $msg)
  {
    if ($this->enableTraceLog)
      $this->result .= "<!-- " . FUBABY_EBAY_PLUGIN_TITLE . " Trace Log: {$msg} -->\r\n";
  }

  protected function errorLog(string $msg)
  {
    error_log(FUBABY_EBAY_PLUGIN_TITLE . ": {$msg}");
  }  
  
  protected function getEndpoint() : string
  {
    $endpointOverride = get_option('fubaby_ebayFelServerOverride');
    return empty($endpointOverride) ? fuEbayApiCall::FEL_ENDPOINT : $endpointOverride;
  }

  protected function getRequestUri() : string
  {
    return $this->requestUri;
  }
  
  public function getResultCount() : int
  {
    return $this->resultCount;
  }
  
  public function forceDeferredLoading()
  {
    $this->deferLoading = fuEbayDeferedLoading::Always;
  }

  public function disableDeferredLoading()
  {
    $this->deferLoading = fuEbayDeferedLoading::Never;
  }

  public function setCachingOptions(int $callCacheExpiry)
  {
    $this->callCacheExpiry = $callCacheExpiry;
  }


  protected function validateArgs(&$message) : bool
  {  
    $this->picWidth = intval($this->picWidth);
    return true;
  }

  private function getCallHash(): string
  {
    return "fuEbay_" . md5($this->requestUri);
  }

  protected function constructRequest()
  { 
    $this->requestUri .= "&ver=" . FUBABY_EBAY_PLUGIN_VER;
    
    $siteHostname = parse_url(get_site_url(), PHP_URL_HOST);
    $this->requestUri .= "&hostname={$siteHostname}";

    $subscriptionKey = get_option('fubaby_ebaySubscriptionKey');
    if (!empty($subscriptionKey))
      $this->requestUri .= "&subkey={$subscriptionKey}";

    $globalId = str_replace("-", "_", get_option('fubaby_ebayGlobalID'));	
    $this->requestUri .= "&globalId={$globalId}";

    $campaignId = get_option('fubaby_ebayCampaignID');
    if (!empty($campaignId) && preg_match_all("/[0-9]/", $campaignId) == 10)
      $this->requestUri .= "&affiliateCampaignId={$campaignId}";

    $referenceId = !empty($this->customId) ? $this->customId : get_option('fubaby_ebayDefCustomID');
    if (!empty($referenceId))
      $this->requestUri .= "&affiliateReferenceId=".urlencode($referenceId)."";
  }

   
  public function call($fromAjax = false)
  {
    $this->result = "";
    $message = "";
    if (!$this->validateArgs($message))
    {
      $this->errorLog($message);
      return "<p>" . FUBABY_EBAY_PLUGIN_TITLE . ": " . __("Bad arguments", "fast-ebay-listings") . "<br/>{$message}</p>";
    }

    if (empty($this->requestUri))
    {
      $this->constructRequest();
      $this->traceLog(sprintf(
        /* translators: %s: the URI */
        __("Request URI: %s", "fast-ebay-listings"),
        $this->requestUri));
    }

    // Fetch the request, either now or defer until after page load
    if ($fromAjax ||
      $this->deferLoading == fuEbayDeferedLoading::Never ||
      ($this->deferLoading == fuEbayDeferedLoading::NotCached && $this->isRequestCached()))
    {
      $result = $this->fetch();
      $this->createListingGrid($result);
    }
    else
    {
      $this->createDeferredLoadingStub();
    }
  
    return $this->result;  
  }

  protected function isRequestCached(): bool
  {
    $callHash = $this->getCallHash();
    $resp = get_transient($callHash);
    $this->traceLog(sprintf(
      /* translators: %s: true/false */
      __("Request cached: %s", "fast-ebay-listings"),
      $resp !== false ? __("True", "fast-ebay-listings") : __("False", "fast-ebay-listings")));

    return $resp !== false;
  }

  protected function fetch()
  {
    // Retrieve cached transient
    $callHash = $this->getCallHash();
    $resp = get_transient($callHash);
    if ($resp !== false && !$this->disableApiCache) 
    {
      $this->traceLog(__("Retrieved pre-cached transient.", "fast-ebay-listings"));
      $result = $this->decodeResponse($resp);
    }
    else
    {
      // No cached transient, so go query remotely.
      $timeStart = microtime(true);
      $resp = wp_remote_retrieve_body(wp_remote_get($this->requestUri));
      if ($resp == null)
      {
        $this->errorLog(__("Failed to query eBay API.", "fast-ebay-listings"));
        return null;
      }

      $result = $this->decodeResponse($resp);
      if (isset($result->statusCode) && $result->statusCode != 200)
      {
        $this->errorLog(
          sprintf(
            /* translators: %d: error status code */
            __("Failed to query eBay API: %d", "fast-ebay-listings"),
            !isset($result->statusCode) ? -1 : $result->statusCode)
          );
        return null;
      }

      // Only save transient on successful fetch
      set_transient($callHash, $resp, $this->callCacheExpiry * MINUTE_IN_SECONDS);
      $timeEnd = microtime(true);
      $this->traceLog(sprintf(
        /* translators: %d: duration passed */
        __("eBay API call took: %d secs.", "fast-ebay-listings"),
        $timeEnd - $timeStart));
    }

    return $result;
  }

  protected function decodeResponse($resp)
  {
    $result = json_decode($resp);
    if (json_last_error() !== JSON_ERROR_NONE)
    {
      $this->errorLog(__("Failed to decode eBay API response.", "fast-ebay-listings"));
      return null;
    }
    return $result;
  }
}