<?php

//////////////////////////////////////////////////////////////////////////
//
// Class to query the eBay Browse API Search function
//
//////////////////////////////////////////////////////////////////////////

require_once __DIR__."/apicall.php";
require_once __DIR__."/presentation.php";

class fuEbayBrowseSearchApiCall extends fuEbayApiCall
{
  const EBAY_LIMIT_MAX = 200;
  const EBAY_CATEGORIES_MAX = 1;

  use fuEbayTable;
  public $isWidget = false;
  
  protected $title = "";
  protected $categoryIds = array();
  protected $aspectName = "";
  protected $aspectValue = "";
  protected $query = "";
  protected $buyingOptions = "";
  protected $sellerType = "";
  protected $sellers = array();
  protected $sortOrder = "";
  
  public $minPrice = -1;
  public $maxPrice = -1;
       
  function __construct(string $title, bool $widget = false) 
  {
    parent::__construct();
    $this->isWidget = $widget;
    $arrangementOptionName = $this->isWidget ? 'fubaby_ebayDefArrangementWidget' : 'fubaby_ebayDefArrangementList';
    $this->arrangement = get_option($arrangementOptionName, fuEbayArrangement::TitleBelow);
    if (empty($this->arrangement)) $this->arrangement = fuEbayArrangement::TitleBelow;
    $this->title = $title;
  }
  
  public function setQuery(string $query)
  {
    $this->query = $query;
  }
  
  // Of the form (and defaults to): <blank> - AUCTION|FIXED_PRICE|BEST_OFFER
  // See https://developer.ebay.com/api-docs/buy/static/ref-buy-browse-filters.html#buyingOptions
  public function setBuyingOptions(string $buyingOptions)
  {
    $this->buyingOptions = $buyingOptions;
  }
  
  // Of the form (and defaults to): <blank> - either. 
  // Can be BUSINESS or INDIVIDUAL, but not both.
  // See https://developer.ebay.com/api-docs/buy/static/ref-buy-browse-filters.html#sellerAccountTypes
  public function setSellerType(string $sellerType)
  {
    switch ($sellerType)
    {
      case fuEbaySellerType::Business: $this->sellerType = fuEbaySellerType::Business; break;
      case fuEbaySellerType::Individual: $this->sellerType = fuEbaySellerType::Individual; break;
      case fuEbaySellerType::Any: $this->sellerType = fuEbaySellerType::Any; break;
      default: $this->sellerType = "";
    }
  }
  

  public function setSellers(string $seller)
  {
    switch (strtolower($seller))
    {
      case "all":
        $this->sellers = array();
        break;
      case "author":
        $seller = get_the_author_meta('ebayuserid');
        // fall through
      default:  
        if (empty($seller))
          $seller = get_option('fubaby_ebayDefSeller');
        $this->sellers = !empty($seller) ? explode(',', $seller) : array();
    }
  }
  
  public function setSortOrder(string $sortOrder)
  {
    global $ebay_map_sort_browseapi;
    
    if (empty($sortOrder))
      return;
    
    if (array_key_exists($sortOrder, $ebay_map_sort_browseapi))
      $this->sortOrder = $sortOrder;  
  }
  
  
  public function setCategories(string $category)
  {
    $this->categoryIds = array();
    if (empty($category))
      $category = get_option('fubaby_ebayDefCategory');

    preg_match('/(?P<id1>\d+)(?P<name> ".*")*[,\s]*(?P<id2>[\d\s]+)*[,\s]*(?P<id3>[\d\s]+)*/', $category, $matches);

    if (isset($matches['id1']))
      $this->categoryIds = array(intval($matches['id1']));
  }
  
  public function setAspect(string $name, string $value)
  {
    $this->aspectName = urlencode($name);
    $this->aspectValue = urlencode($value);
  }

  protected function validateArgs(&$message) : bool
  {
    if (!parent::validateArgs($message)) return false;
    self::validate();

    // Validate Pic Width
    if ($this->picWidth <= 0) 
      $this->picWidth = get_option($this->isWidget ? 'fubaby_ebayPicWidthWidget' : 'fubaby_ebayPicWidthList', 225);	

    // Validate Limits set by Browse Item_Summary/Search API call
    if ($this->limit > fuEbayBrowseSearchApiCall::EBAY_LIMIT_MAX) $this->limit = fuEbayBrowseSearchApiCall::EBAY_LIMIT_MAX;

    // Validate sorting
    if (empty($this->sortOrder)) $this->sortOrder = get_option('fubaby_ebayDefSort', fuEbaySortOrder::BestMatch);

    // Validate categories
    if (is_countable($this->categoryIds) && count($this->categoryIds) > fuEbayBrowseSearchApiCall::EBAY_CATEGORIES_MAX)
    {
      $message = sprintf(
          /* translators: %d: max number of categories */
          __("Too many category IDs, max = %d", "fast-ebay-listings"), 
          fuEbayBrowseSearchApiCall::EBAY_CATEGORIES_MAX);
      return false;
    }
 
    // Validate query or category - one is required.
    if (empty($this->query) && is_countable($this->categoryIds) && count($this->categoryIds) == 0)
    {
      $message = __("Search query and/or category ID must be specified", "fast-ebay-listings");
      return false;
    }

    // Validate min/max prices
    $this->minPrice = floatval($this->minPrice);
    if ($this->minPrice < 0) $this->minPrice = 0;
    $this->maxPrice = floatval($this->maxPrice);
    if ($this->maxPrice < 0) $this->maxPrice = 0;
    if ($this->minPrice > 0 && $this->maxPrice > 0 && $this->minPrice > $this->maxPrice)
    {
      $message = __("Minimum price must be less than maximum price.", "fast-ebay-listings");
      return false;
    }

    // Validate seller type
    if (empty($this->sellerType))
      $this->sellerType = get_option('fubaby_ebayDefSellerType', ''); // blank, use settings default.
    if ($this->sellerType != fuEbaySellerType::Business && $this->sellerType != fuEbaySellerType::Individual)
      $this->sellerType = ""; // not valid, set to any.

    // Validate Buying Options
    if ($this->buyingOptions != fuEbayBuyingOptions::FixedPrice && $this->buyingOptions != fuEbayBuyingOptions::Auction && $this->buyingOptions != fuEbayBuyingOptions::BestOffer)
    {
      $this->buyingOptions = "";
    }

    return true;
  }    


  protected function constructRequest()
  {
    // Round limit up to nearest 12 (to a max of 200). More likely to hit cache on future variations
    $querylimit = ceil($this->limit / 12) * 12;
    $querylimit = $querylimit > 200 ? 200 : $querylimit;

    $queryParams = array();

    // There must be either a query or a category. 
    if (!empty($this->query))
      $queryParams["q"] = $this->query;
    if (is_countable($this->categoryIds) && count($this->categoryIds) > 0)
      $queryParams["categoryIds"] = implode(',', $this->categoryIds);

    // Other request params
    if (!empty($this->buyingOptions))
      $queryParams["buyingOptions"] = $this->buyingOptions;
    if (!empty($this->sellerType))
      $queryParams["sellerAccountTypes"] = $this->sellerType;
    $queryParams["limit"] = $querylimit;
    $queryParams["sort"] = $this->sortOrder;
    if (is_countable($this->sellers) && count($this->sellers) > 0)
      $queryParams["sellers"] = implode("|", $this->sellers);
    if ($this->minPrice > 0)
      $queryParams["minPrice"] = $this->minPrice;
    if ($this->maxPrice > 0)
      $queryParams["maxPrice"] = $this->maxPrice;

    $this->requestUri = $this->getEndpoint() . "search?" . http_build_query($queryParams);

    parent::constructRequest();

  }

  protected function createListingGrid($resp)
  {
    global $ebay_globalid_siteurls;
    $fixedHeight = $this->slideShow != fuEbaySlideShowStyle::None;

    $displayFieldsOptionName = $this->isWidget ? 'fubaby_ebayDefDisplayFieldsWidget' : 'fubaby_ebayDefDisplayFieldsList';
    $displayFields = count($this->displayFields) > 0 ? $this->displayFields : get_option($displayFieldsOptionName, fuEbayDisplayFields::ListingDefault);

    if ($resp != null)
    {
      $titleUrl = isset($resp->itemSummaries) ? $resp->itemSummaries[0]->itemAffiliateWebUrl : $ebay_globalid_siteurls[$this->globalId];
  	  $this->genTableStart($titleUrl);
  
  	  if (isset($resp->itemSummaries))
  	  {
        $this->resultCount = count($resp->itemSummaries);
        $count = 0;
  		  foreach($resp->itemSummaries as $item)
  		  {
          $ebayItem = $this->initEbayListing($item);
          $this->genTableItem($ebayItem->GetListingAsHtml($this->picWidth, $displayFields, $fixedHeight));
          if (++$count >= $this->limit)
            break;
  		  }
  	  }

  	  if ($this->resultCount == 0)
  		  $this->result .= "<p>" . get_option('fubaby_ebayEmptySearchMsg', __('No items found', "fast-ebay-listings")) . "</p>";
  	  
   	  $this->genTableEnd();
    }      
  }

  // Init eBay Listing class from a search result item from API call.
  protected function initEbayListing($item)
  {
    global $ebay_globalid_currencies;
  
    $ebayItem = $this->createEbayListing();
    $ebayItem->Currency = $ebay_globalid_currencies[$this->globalId];
    $ebayItem->Arrangement = $this->arrangement;
    
    $ebayItem->ItemId = (string)$item->itemId;
    $ebayItem->Title = (string)$item->title;
    $ebayItem->ShortDescription = isset($item->shortDescription) ? (string)$item->shortDescription : null;
    $ebayItem->Url = (string)$item->itemAffiliateWebUrl;
    $ebayItem->Sold = false;
    if (isset($item->thumbnailImages))
    {
      $ebayItem->PicHuge = (string)$item->thumbnailImages[0]->imageUrl;      // 1000px
      $ebayItem->PicLarge = (string)$item->thumbnailImages[0]->imageUrl;     // 1000px
    }
    if (isset($item->image->imageUrl))
    {
      $ebayItem->GalleryLarge = (string)$item->image->imageUrl;     // 225px
      $ebayItem->GalleryMedium = (string)$item->image->imageUrl;     // 225px
      $ebayItem->GallerySmall = (string)$item->image->imageUrl;     // 225px
    }
    
    $ebayItem->AuctionBids = isset($item->bidCount) ? (int)$item->bidCount : null;
    $ebayItem->EndTime = isset($item->itemEndDate) ? (string)$item->itemEndDate : null;
    $ebayItem->Active = isset($item->itemEndDate) ? strtotime($ebayItem->EndTime) > time() : true;
    $ebayItem->Location = isset($item->itemLocation->city) ? "{$item->itemLocation->city}, {$item->itemLocation->country}" : $item->itemLocation->country;
    $ebayItem->Feedback = (int)$item->seller->feedbackScore;
    $ebayItem->FeedbackPercent = isset($item->seller->feedbackPercentage) ? intval($item->seller->feedbackPercentage) : 100;
    $ebayItem->priorityListing = isset($item->priorityListing) && $item->priorityListing == "true";
    $ebayItem->Condition = isset($item->condition) ? (string)$item->condition : null;
  
    if (isset($item->buyingOptions))
    {
      if (in_array(fuEbayBuyingOptions::Auction, $item->buyingOptions))
      {
        $ebayItem->Price = $item->currentBidPrice->value;
        $ebayItem->PriceConv = $item->currentBidPrice->value;

        if (in_array(fuEbayBuyingOptions::FixedPrice, $item->buyingOptions))
        {
          $ebayItem->PriceBIN = $item->price->value;
          $ebayItem->PriceBINConv = $item->price->value;
          $ebayItem->ListingType = "AuctionWithBIN"; // Auction + Fixed price = AuctionWithBIN
        }
        else
          $ebayItem->ListingType = "Auction";
      }
      
      // Auctions with BIN/Best Offers
      if ((in_array(fuEbayBuyingOptions::FixedPrice, $item->buyingOptions) && !in_array(fuEbayBuyingOptions::Auction, $item->buyingOptions)) ||
        in_array(fuEbayBuyingOptions::ClassifiedAd, $item->buyingOptions))
      {
        $ebayItem->Price = $ebayItem->PriceBIN = $item->price->value;
        $ebayItem->PriceConv = $ebayItem->PriceBINConv = $item->price->value;
        $ebayItem->ListingType = "FixedPrice";
      }
      
      if (in_array(fuEbayBuyingOptions::BestOffer, $item->buyingOptions))
        $ebayItem->BINBestOffer = true;
    }

    if (isset($item->seller->username))
    {
      if (isset($item->seller->sellerAccountType) && $item->seller->sellerAccountType == fuEbaySellerType::Business)
        $ebayItem->StoreName = (string)$item->seller->username;
      else
        $ebayItem->SellerName = (string)$item->seller->username;
    }
    
    return $ebayItem;
  }
        

  protected function createDeferredLoadingStub()
  {
    global $ebay_globalid_siteurls;
    global $fuEbayTableSlideId;
    
    $stubId = "fu_ebay_" . parent::$id++;

    $this->genTableStart($ebay_globalid_siteurls[$this->globalId]);
    $this->result .= __("Loading...", "fast-ebay-listings");
    $this->genTableEnd();

    // Recreate result with above embedded within it. Thus no concatenate onto result.
    $this->result = '<div id="'.$stubId.'">'.$this->result.'</div>
    <script>
    var $content_'.$stubId.' = jQuery("#'.$stubId.'");
    if (!fu_ebay_is_bot_useragent){
    jQuery(document).on("load", jQuery.ajax({
      type : "post",
      url : "'.admin_url('admin-ajax.php').'",
      data : {
          action : "fu_ebay_load_search",
          fu_ebay_widget : '.($this->isWidget ? "1" : "0").',
          fu_ebay_title : '.json_encode($this->title).',
          fu_ebay_query : '.json_encode($this->query).',
          fu_ebay_limit : "'.$this->limit.'",
          fu_ebay_sort : "'.$this->sortOrder.'",
          fu_ebay_categoryIds : '.json_encode(implode(',', $this->categoryIds)).',
          fu_ebay_sellers : '.json_encode(implode(',', $this->sellers)).',
          fu_ebay_sellerType : "'.$this->sellerType.'",
          fu_ebay_buyingOptions : "'.$this->buyingOptions.'",
          fu_ebay_minPrice : "'.$this->minPrice.'",
          fu_ebay_maxPrice : "'.$this->maxPrice.'",
          fu_ebay_picwidth : "'.$this->picWidth.'",
          fu_ebay_cols : "'.$this->columns.'",
          fu_ebay_rows : "'.$this->rows.'",
          fu_ebay_slideshow : "'.$this->slideShow.'",
          fu_ebay_slides : "'.$this->slides.'",
          fu_ebay_slideId : "'.$fuEbayTableSlideId.'",
          fu_ebay_customId : "'.$this->customId.'",
          nonce : "'.wp_create_nonce("shortcode-search").'",
      },
      success : function( response ) {
          $content_'.$stubId.'.html( response );
      },
      error: function(e) {
        console.log("'.FUBABY_EBAY_PLUGIN_TITLE.': '.__("deferred loading error:", "fast-ebay-listings").' " + e);
      }
    }));
    }
    </script>
    ';
  }


}

// Main AJAX entry function
function fu_ebay_load_search() 
{
  global $fuEbayTableSlideId;
  try
  {
    if (!check_ajax_referer('shortcode-search', 'nonce', false)) 
    {
      $this->errorLog('Nonce check failed');
      die;
    }

    $apicall = new fuEbayBrowseSearchApiCall(stripslashes($_POST['fu_ebay_title']));
    if (isset($_POST['fu_ebay_widget']) && $_POST['fu_ebay_widget'])
      $apicall->isWidget = true;
    if (isset($_POST['fu_ebay_query'])) 
      $apicall->setQuery(stripslashes($_POST['fu_ebay_query']));
    if (isset($_POST['fu_ebay_sort'])) 
      $apicall->setSortOrder($_POST['fu_ebay_sort']);
    if (isset($_POST['fu_ebay_categoryIds'])) 
      $apicall->setCategories(stripslashes($_POST['fu_ebay_categoryIds']));
    if (isset($_POST['fu_ebay_sellers'])) 
      $apicall->setSellers(stripslashes($_POST['fu_ebay_sellers']));
    if (isset($_POST['fu_ebay_sellerType'])) 
      $apicall->setSellerType($_POST['fu_ebay_sellerType']);
    if (isset($_POST['fu_ebay_buyingOptions'])) 
      $apicall->setBuyingOptions($_POST['fu_ebay_buyingOptions']);
    if (isset($_POST['fu_ebay_minPrice'])) 
      $apicall->minPrice = $_POST['fu_ebay_minPrice'];
    if (isset($_POST['fu_ebay_maxPrice'])) 
      $apicall->maxPrice = $_POST['fu_ebay_maxPrice'];
    if (isset($_POST['fu_ebay_picwidth'])) 
      $apicall->picWidth = $_POST['fu_ebay_picwidth'];
    if (isset($_POST['fu_ebay_cols']) && isset($_POST['fu_ebay_rows'])) 
      $apicall->setTableSize($_POST['fu_ebay_cols'], $_POST['fu_ebay_rows']);
    if (isset($_POST['fu_ebay_slideshow']) && isset($_POST['fu_ebay_slides'])) 
      $apicall->setSlideShow($_POST['fu_ebay_slideshow'], $_POST['fu_ebay_slides']); 
    if (isset($_POST['fu_ebay_customId'])) $apicall->customId = $_POST['fu_ebay_customId'];

    // override slide ids to avoid clashing with other slides on pages.
    $fuEbayTableSlideId = intval($_POST['fu_ebay_slideId']);

    echo $apicall->call(true);

  } catch(TypeError $e)
  {
    echo $e->getMessage();
  }
  die();
}

add_action( 'wp_ajax_fu_ebay_load_search', 'fu_ebay_load_search' );
add_action( 'wp_ajax_nopriv_fu_ebay_load_search', 'fu_ebay_load_search' );