<?php

//////////////////////////////////////////////////////////////////////////
//
// Class to query an eBay seller's feedback
//
//////////////////////////////////////////////////////////////////////////

require_once __DIR__."/apicall.php";

class fuEbayFeedbackApiCall extends fuEbayApiCall
{  
  use fuEbayTable;

  protected $seller;
  public $feedbackLimit = 10;
  
  function __construct($seller) 
  {
    parent::__construct();
    $this->setSeller($seller);
    $this->callCacheExpiry = FUBABY_EBAY_CALLCACHEEXPIRYFEEDBACK;
}

  public function setSeller(string $seller)
  {
    switch (strtolower($seller))
    {
      case "all":
        $this->seller = "";
        break;
      case "author":
        $seller = get_the_author_meta('ebayuserid');
        // fall through
      default:  
        if (empty($seller))
          $seller = get_option('fubaby_ebayDefSeller');
        $this->seller = !empty($seller) ? explode(',', $seller)[0] : "";
    }
  }
  protected function validateArgs(&$message) : bool
  {
    if (!parent::validateArgs($message)) return false;
    
    if (empty($this->seller))
    {
      $message = __("Seller ID must not be empty.", "fast-ebay-listings");
      return false;
    }

    $this->feedbackLimit = intval($this->feedbackLimit);
    if ($this->feedbackLimit < 0)
    {
      $message = __("Feedback limit must be a positive integer.", "fast-ebay-listings");
      return false;
    }

    return true;
  } 

  protected function constructRequest()
  {
    $queryParams = array(
      "id" => $this->seller,
      "limit" => $this->feedbackLimit
    );
    $this->requestUri = $this->getEndpoint() . "feedback?" . http_build_query($queryParams);
    parent::constructRequest();
  }

  protected function createListingGrid($resp)
  {
    global $fu_feedback_rating_details, $ebay_globalid_siteurls;

    if ($resp != null && isset($resp->GetUserResponse) && isset($resp->GetUserResponse->Ack) && $resp->GetUserResponse->Ack == "Success") 
    {
      $getUser = $resp->GetUserResponse;
      $getStore = $resp->GetStoreResponse;
      $getFeedback = $resp->GetFeedbackResponse;

      $hasStore = $getStore->Ack == "Success";
      $url = $hasStore ? $getStore->Store->URL : "{$ebay_globalid_siteurls[$this->globalId]}usr/{$getUser->User->UserID}";
      if (isset($resp->AffiliateURLQueryParam)) $url .= "?" . $resp->AffiliateURLQueryParam;

      $this->title = $hasStore ? $getStore->Store->Name : $getUser->User->UserID;
      $this->title .= " ({$getFeedback->FeedbackScore})";
      $this->genTableStart($url);

      //---------------------------------------------------------
      // Seller feedback summary
      $this->result .= "<div class=\"fu_ebay_results_row\"><div class=\"fu_ebay_results_cell fu_ebay_results_cell_padding\" >\r\n";

      $negFeedbackCount = (double)$this->getFeedbackPeriodCount($getFeedback->FeedbackSummary->NegativeFeedbackPeriodArray->FeedbackPeriod, 365);
      $totalFeedbackCount = (double)$this->getFeedbackPeriodCount($getFeedback->FeedbackSummary->TotalFeedbackPeriodArray->FeedbackPeriod, 365);
      $posFeedbackPercent = $totalFeedbackCount > 0 ? $negFeedbackCount / $totalFeedbackCount : 0;
      $this->result .= "<div class=\"fu_ebay_feedback_summary\">";
      $this->result .= sprintf(
        /* Translators: %f: feedback percentage */
        __("Positive Feedback: %.1f%% (last 12 months)", "fast-ebay-listings"), 
        (1 - $posFeedbackPercent) * 100);
      $this->result .= "</div>\r\n";

      // Seller eBay membership period
      $registrationDate = strtotime($getUser->User->RegistrationDate);
      $this->result .= "<div class=\"fu_ebay_feedback_summary\">";
      $this->result .= sprintf(
        /* Translators: %1$s: date, %2$s: Country */
        __('Member since: %1$s in %2$s', "fast-ebay-listings"), 
        date_i18n("F j, Y", $registrationDate),
        $getUser->User->Site);
      $this->result .= "</div>\r\n";

      if ($hasStore)
      {
        $this->result .= "<div class=\"fu_ebay_feedback_summary\">";
        $this->result .= fuEbayUtils::GetEbayLink(
          $url,
          __("Visit Shop", "fast-ebay-listings") . " " . $getStore->Store->Name,
          $getUser->User->UserID,
          "&rarr; " . __("Visit Shop", "fast-ebay-listings"));    
        $this->result .= "</div>\r\n";
      }

      $this->result .= "</div></div>\r\n";

      //---------------------------------------------------------
      // Feedback Rating
      $this->result .= "<div class=\"fu_ebay_results_row\"><div class=\"fu_ebay_results_cell fu_ebay_results_cell_padding fu_ebay_results_cell_paddingtop\" >\r\n";
      $this->result .= "<span class=\"fu_ebay_feedback_header\">" . __('Feedback Ratings', "fast-ebay-listings") . "</span>";
      $this->result .= "</div></div>\r\n";

      $this->result .= "<div class=\"fu_ebay_results_row\"><div class=\"fu_ebay_results_cell fu_ebay_results_cell_padding\" >\r\n";
      $this->result .= "<div class=\"fu_ebay_feedback_rating_container\">\r\n";
    
      // Feedback ratings (average rating details)
      $this->result .= "<div class=\"fu_ebay_feedback_rating_table\">\r\n";
      $averageRatingSummary = $this->getFeedbackAverageRatingSummaryForYear($getFeedback->FeedbackSummary->SellerRatingSummaryArray->AverageRatingSummary);
      foreach ($averageRatingSummary->AverageRatingDetails as $detail)
      {
        $this->result .= "<div class=\"fu_ebay_feedback_detail\">";
        $this->result .= $this->getFeedbackStars($detail->Rating) . " ";
        $this->result .= $detail->RatingCount . " ";
        $this->result .= $fu_feedback_rating_details[$detail->RatingDetail];
        $this->result .= "</div>\r\n";

      }
      $this->result .= "</div>\r\n";  // fu_ebay_feedback_rating_table
      
      // Feedback ratings
      $this->result .= "<div class=\"fu_ebay_feedback_rating_table\">\r\n";

      $this->result .= "<div class=\"fu_ebay_feedback_detail\">" . $this->getRatingIcon("Positive");
      $this->result .= sprintf(
        /* Translators: %d: Feedback count */
        __(' %d Positive', "fast-ebay-listings"), 
        $this->getFeedbackPeriodCount($getFeedback->FeedbackSummary->PositiveFeedbackPeriodArray->FeedbackPeriod, 365));
      $this->result .= "</div>\r\n";

      $this->result .= "<div class=\"fu_ebay_feedback_detail\">" . $this->getRatingIcon("Neutral");
      $this->result .= sprintf(
        /* Translators: %d: Feedback count */
        __(' %d Neutral', "fast-ebay-listings"), 
        $this->getFeedbackPeriodCount($getFeedback->FeedbackSummary->NeutralFeedbackPeriodArray->FeedbackPeriod, 365));
      $this->result .= "</div>\r\n";

      $this->result .= "<div class=\"fu_ebay_feedback_detail\">" . $this->getRatingIcon("Negative");
      $this->result .= sprintf(
        /* Translators: %d: Feedback count */
        __(' %d Negative', "fast-ebay-listings"), 
        $this->getFeedbackPeriodCount($getFeedback->FeedbackSummary->NegativeFeedbackPeriodArray->FeedbackPeriod, 365));
      $this->result .= "</div>\r\n";
        
      $this->result .= "</div>\r\n";  // fu_ebay_feedback_rating_table
      $this->result .= "</div>\r\n";  // fu_ebay_feedback_rating_container;

      $this->result .= "</div>\r\n"; // fu_ebay_results_cell
      $this->result .= "</div>\r\n"; // fu_ebay_results_row


      //---------------------------------------------------------
      // Feedback Details
      if ($this->feedbackLimit > 0)
      {
        $this->result .= "<div class=\"fu_ebay_results_row\"><div class=\"fu_ebay_results_cell fu_ebay_results_cell_padding fu_ebay_results_cell_paddingtop\" >\r\n";
        $this->result .= "<span class=\"fu_ebay_feedback_header\">" . __('Feedback', "fast-ebay-listings") . "</span>";
        $this->result .= "</div></div>\r\n";

        $feedbackCount = 0;
        foreach ($getFeedback->FeedbackDetailArray->FeedbackDetail as $detail)
        {
          $this->result .= "<div class=\"fu_ebay_results_row\"><div class=\"fu_ebay_results_cell fu_ebay_results_cell_padding fu_ebay_results_cell_bottomborder\" >\r\n";
          $this->result .= "<div class=\"fu_ebay_feedback_detailleft\">" . $this->getRatingIcon($detail->CommentType) . "</div>";
          $this->result .= "<div class=\"fu_ebay_feedback_detailright\">";
          $this->result .= "<span class=\"fu_ebay_feedback_detail_comment\">" . $detail->CommentType . " " . $detail->CommentText ."</span>";
          if (isset($detail->ItemTitle))
          {
            $itemUrl = "{$ebay_globalid_siteurls[$this->globalId]}itm/{$detail->ItemID}";
            if (isset($resp->AffiliateURLQueryParam)) $itemUrl .= "?" . $resp->AffiliateURLQueryParam;

            $this->result .= "<br/><span class=\"fu_ebay_feedback_item\">" . fuEbayUtils::GetEbayLink($itemUrl, __("View Item"), $detail->ItemID, $detail->ItemTitle) ."</span>";
          }
          $this->result .= "</div></div></div>\r\n";
          if (++$feedbackCount >= $this->feedbackLimit)
            break;
        }
      }

      $this->genTableEnd();
    }
    else 
    {
      $this->result .= "<p>" . __('Seller not found', "fast-ebay-listings") . "</p>";
    }
  }

  // Some help functions
  private function getFeedbackPeriodCount($feedbackPeriod, $periodInDays) : int
  {
    foreach ($feedbackPeriod as $element ) 
    {
      if ($element->PeriodInDays == $periodInDays)
          return $element->Count;
    }
    return 0;
  }

  private function getFeedbackAverageRatingSummaryForYear($averageRatingSummary)
  {
    if (is_array($averageRatingSummary))
    {
      foreach ($averageRatingSummary as $element )
      {
        if ($element->FeedbackSummaryPeriod == "FiftyTwoWeeks")
          return $element;
      }
    }

    return $averageRatingSummary;
  }

  private function getRatingIcon($rating)
  {
    if ($rating == "Positive")
      return '<svg class="fu_ebay_feedback_icon" viewBox="0 0 23 23" height="24" width="24" aria-label="Positive feedback rating" role="img"><path fill="#05823F" d="M11.017 0C4.935 0 0 4.929 0 11.017c0 6.082 4.935 11.016 11.017 11.016s11.016-4.934 11.016-11.016C22.033 4.929 17.099 0 11.017 0zm7.131 11.665c0 .714-.579 1.296-1.299 1.296h-3.885v3.888c0 .712-.586 1.296-1.299 1.296h-1.299a1.297 1.297 0 0 1-1.293-1.296v-3.888H5.182a1.296 1.296 0 0 1-1.293-1.296v-1.299c0-.713.579-1.296 1.293-1.296h3.891V5.182c0-.713.579-1.295 1.293-1.295h1.299c.713 0 1.299.582 1.299 1.295V9.07h3.884c.719 0 1.299.583 1.299 1.296v1.299h.001z"></path></svg>';
    if ($rating == "Neutral")
      return '<svg class="fu_ebay_feedback_icon" viewBox="0 0 23 23" height="24" width="24" aria-label="Neutral feedback rating" role="img"><path fill="#6A6A6A" d="M11.016 0C4.933 0 0 4.93 0 11.015 0 17.1 4.932 22.031 11.016 22.031S22.032 17.1 22.032 11.015 17.101 0 11.016 0zm4.131 13.769c0 .76-.617 1.376-1.377 1.376H8.262c-.76 0-1.377-.616-1.377-1.376V8.26c0-.76.616-1.377 1.377-1.377h5.508c.76 0 1.377.617 1.377 1.377v5.509z"></path></svg>';
    if ($rating == "Negative")
      return '<svg class="fu_ebay_feedback_icon" viewBox="0 0 22 22" height="24" width="24" aria-label="Negative feedback rating" role="img"><path fill="#E0103A" d="M10.969 0C4.911 0 0 4.911 0 10.969s4.911 10.969 10.969 10.969 10.969-4.91 10.969-10.969C21.938 4.912 17.027 0 10.969 0zm4.113 12.339H6.854a1.37 1.37 0 1 1 0-2.74h8.227a1.37 1.37 0 1 1 0 2.74h.001z"></path></svg>';
    return "?";
  }

  private function getFeedbackStars($rating)
  {
    return '<span class="fu_ebay_feedback_stars"><span style="width: ' . ($rating * 16) . 'px"></span></span>';
  }
  
  // Deferred loading functions
  protected function createDeferredLoadingStub()
  {
    $stubId = "fu_ebay_" . parent::$id++;

    $this->result = '
    <div id="'.$stubId.'">' . __("Loading...", "fast-ebay-listings") . '</div>
    <script>
    var $content'.$stubId.' = jQuery("#'.$stubId.'");
    jQuery(document).on("load", jQuery.ajax({
      type : "post",
      url : "'.admin_url('admin-ajax.php').'",
      data : {
          action : "fu_ebay_load_feedback",
          fu_ebay_seller : "'.$this->seller.'",
          fu_ebay_limit : "'.$this->feedbackLimit.'",
          nonce : "'.wp_create_nonce("shortcode-feedback").'",
      },
      success : function( response ) {
          $content'.$stubId.'.html( response );
      },
      error: function(e) {
        console.log("'.FUBABY_EBAY_PLUGIN_TITLE.': '.__("deferred loading error:", "fast-ebay-listings").' " + e);
      }
    }));
    </script>
    ';

  }
   
}

// Main AJAX entry function
function fu_ebay_load_feedback() 
{
  if (!check_ajax_referer('shortcode-feedback', 'nonce', false)) 
  {
    $this->errorLog('Nonce check failed');
    die;
  }

  $apicall = new fuEbayFeedbackApiCall($_POST['fu_ebay_seller']);
  $apicall->feedbackLimit = $_POST['fu_ebay_limit'];
  echo $apicall->call(true);
  die();
}

add_action( 'wp_ajax_fu_ebay_load_feedback', 'fu_ebay_load_feedback' );
add_action( 'wp_ajax_nopriv_fu_ebay_load_feedback', 'fu_ebay_load_feedback' );